module Deadlift
  class Base
    attr_reader :barbell

    def self.barbell(params = {})
      Deadlift::Barbell::Default.new(params)
    end

    def self.validate(*messages)
      @validations ||= []
      validations.concat messages
    end

    def self.validations
      @validations
    end

    def self.get_default_trainers
      Array(@default_trainers)
    end

    def self.default_trainers(*trainers)
      @default_trainers = trainers
    end

    def initialize(params)
      klass = self.class
      @params = params
      @barbell = klass.barbell(@params)
      klass.get_default_trainers.each{|trainer| add_trainer(trainer)}
    end

    def add_trainer(*trainers)
      self.trainers.concat trainers
      self
    end

    def perform_and_report_for_valid(&block)
      if barbell_approved?
        yield
        report :success
      else
        report :failure
      end
    end

    def trainers
      @trainers ||= []
    end

    def report(type)
      trainers.each do |trainer|
        trainer.public_send("#{type}", performance)
      end
      nil
    end

    def report_progress(performance)
      trainers.each do |trainer|
        trainer.public_send(:progress, performance)
      end
      nil
    end

    def barbell_approved?
      return false unless barbell.approved?
      Array(self.class.validations).each do |message|
        self.send message
      end
      Array(barbell.errors).empty?
    end

    def performance
      @performance ||= Deadlift::Performance.new(barbell, result, interaction)
    end

    # Override if needed
    def result
      NullResult.new
    end

    def interaction
      self.class.name.demodulize.underscore.to_sym
    end

    class NullResult; end
  end
end
